import { Subject } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 * @param {?=} dataKey
 * @param {?=} routeKey
 * @param {?=} ctr
 * @return {?}
 */
function FromRouteData(dataKey = null, routeKey = 'route', ctr = null) {
    return (/**
     * @param {?} target
     * @param {?} propertyName
     * @return {?}
     */
    (target, propertyName) => {
        if (dataKey == null) {
            dataKey = propertyName;
        }
        Object.defineProperty(target, propertyName, {
            /**
             * @return {?}
             */
            get() {
                /** @type {?} */
                const route = (/** @type {?} */ (this[routeKey]));
                if (route == null) {
                    console.error(`Property ${routeKey} is not "ActivatedRoute". Maybe you forgot to inject it`);
                }
                /** @type {?} */
                const data = route.snapshot.data;
                if (data != null && dataKey in data) {
                    if (ctr != null) {
                        return new ctr(data[dataKey]);
                    }
                    else {
                        return data[dataKey];
                    }
                }
                else {
                    return null;
                }
            },
            /**
             * @return {?}
             */
            set() {
                throw new Error('Property is readonly');
            }
        });
    });
}
/**
 * @template T
 * @param {?=} property
 * @param {?=} routeKey
 * @param {?=} ctr
 * @return {?}
 */
function MapRouteParams(property = null, routeKey = 'route', ctr = null) {
    return (/**
     * @param {?} target
     * @param {?} propertyName
     * @return {?}
     */
    (target, propertyName) => {
        Object.defineProperty(target, propertyName, {
            /**
             * @return {?}
             */
            get() {
                /** @type {?} */
                const route = (/** @type {?} */ (this[routeKey]));
                if (route == null) {
                    console.error(`Property ${routeKey} is not "ActivatedRoute". Maybe you forgot to inject it`);
                }
                /** @type {?} */
                const params = route.snapshot.params;
                if (params != null) {
                    if (property == null) {
                        if (ctr == null) {
                            return params;
                        }
                        else if (ctr === 'Array') {
                            throw new Error('Field "property" should be defined to take array from params');
                        }
                        else {
                            return new ctr((/** @type {?} */ (params)));
                        }
                    }
                    if (params[property] == null) {
                        return null;
                    }
                    if (ctr == null) {
                        return params[property];
                    }
                    else if (ctr === 'Array') {
                        return JSON.parse(`[${params[property]}]`);
                    }
                    else {
                        return new ctr((/** @type {?} */ (params)));
                    }
                }
            },
            /**
             * @return {?}
             */
            set() {
                throw new Error('Property is readonly');
            }
        });
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const mapObjectToAttributes = (/**
 * @param {?} obj
 * @return {?}
 */
(obj) => {
    /** @type {?} */
    const attributes = Object.keys(obj).map((/**
     * @param {?} attr
     * @return {?}
     */
    attr => `${attr}="${obj[attr]}"`));
    return attributes.join(' ');
});
/** @type {?} */
const mapObjectToStyles = (/**
 * @param {?} obj
 * @return {?}
 */
(obj) => {
    /** @type {?} */
    const styles = Object.keys(obj).map((/**
     * @param {?} style
     * @return {?}
     */
    style => `${style}: ${obj[style]}`));
    return `style="${styles.join('; ')}"`;
});
/** @type {?} */
const build = (/**
 * @param {?} template
 * @param {?=} styles
 * @param {?=} attrs
 * @param {?=} classes
 * @return {?}
 */
(template, styles = {}, attrs = {}, classes = []) => `<div class="${classes.join(' ')}" ${mapObjectToStyles(styles)} ${mapObjectToAttributes(attrs)}>${template}</div>`);
class Wrapper {
    /**
     * @param {?} _template
     */
    constructor(_template) {
        this._template = _template;
        this._classes = [];
        this._flexDefaults = { flex: true, alignment: 'stretch', justification: 'flex-start', direction: 'column', fix: true };
        this.withStyles = (/**
         * @template THIS
         * @this {THIS}
         * @param {?} styles
         * @return {THIS}
         */
        (styles) => {
            this._stylesObj = Object.assign({}, this._stylesObj, styles);
            return this;
        });
        this.withAttrs = (/**
         * @template THIS
         * @this {THIS}
         * @param {?} attrs
         * @return {THIS}
         */
        (attrs) => {
            this._attrsObj = Object.assign({}, this._attrsObj, attrs);
            return this;
        });
        this.withWidth = (/**
         * @template THIS
         * @this {THIS}
         * @param {?} width
         * @return {THIS}
         */
        (width) => {
            this._stylesObj = Object.assign({}, this._stylesObj, { width });
            return this;
        });
        this.withHeight = (/**
         * @template THIS
         * @this {THIS}
         * @param {?} height
         * @return {THIS}
         */
        (height) => {
            this._stylesObj = Object.assign({}, this._stylesObj, { height });
            return this;
        });
        this.withMaxWidth = (/**
         * @template THIS
         * @this {THIS}
         * @param {?} width
         * @return {THIS}
         */
        (width) => {
            this._stylesObj = Object.assign({}, this._stylesObj, { 'max-width': width });
            return this;
        });
        this.withMaxHeight = (/**
         * @template THIS
         * @this {THIS}
         * @param {?} height
         * @return {THIS}
         */
        (height) => {
            this._stylesObj = Object.assign({}, this._stylesObj, { 'max-height': height });
            return this;
        });
        this.withFlex = (/**
         * @template THIS
         * @this {THIS}
         * @param {?=} params
         * @return {THIS}
         */
        (params = this._flexDefaults) => {
            this._attrsObj = Object.assign({}, this._attrsObj, this._flexDefaults, params);
            return this;
        });
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} className
     * @return {THIS}
     */
    withClass(className) {
        (/** @type {?} */ (this))._classes.push(className);
        return (/** @type {?} */ (this));
    }
    /**
     * @return {?}
     */
    build() {
        return build(this._template, this._stylesObj, this._attrsObj, this._classes);
    }
}
/** @type {?} */
const wrapper = (/**
 * @param {?} template
 * @return {?}
 */
(template) => new Wrapper(template));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class FlexComponent {
    constructor() {
        this._classes = 'component';
        this.destroySubject = new Subject();
        this.destroy = this.destroySubject.asObservable();
        this.flex = true;
        this.fix = true;
        this.alignment = 'stretch';
        this.justification = 'flex-start';
        this.direction = 'column';
        this.wrap = 'nowrap';
        this.additionClasses = [];
    }
    /**
     * @return {?}
     */
    get classes() {
        return this._classes;
    }
    /**
     * @param {?} v
     * @return {?}
     */
    set classes(v) {
        this._classes = `component ${v} ${this.additionClasses.join(' ')}`;
    }
    // tslint:disable-next-line: use-life-cycle-interface
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroySubject.next();
        this.destroySubject.complete();
    }
}

export { FlexComponent, FromRouteData, MapRouteParams, Wrapper, wrapper };
//# sourceMappingURL=dev-stream-angular-utils.js.map
